<?php

/**
 * @file
 * Page callbacks for Avatar approval.
 */

/**
 * (Internal) Return operations dropdown menu
 *
 * This is were we create the approval dropdown menu items
 * and the callbacks
 *
 * @return
 *     An associative array to be used with Drupal Menu API
 */
function _avatarapproval_operations() {
  $operations = array(
    'approve' => array(
      'text' => t('Approve the selected avatars'),
      'callback' => 'avatarapproval_do_approval',
    ),
    'disapprove' => array(
      'text' => t('Disapprove the selected avatars'),
      'callback' => 'avatarapproval_do_disapprove',
    ),
  );
  // We only want to give deletion power to priviliged people
  if (user_access('delete avatars')) {
    $operations['delete'] = array(
      'text' => t('Delete the selected avatars'),
      'callback' => 'avatarapproval_do_delete',
    );
  }
  return $operations;
}

/**
 * (Internal) Return a themed list of usernames
 *
 * Return a themed list of usernames associated to an
 * avatar, given its md5
 *
 * @param $md5
 *     The md5 of the avatar
 * @return
 *     theme() output consisting in a list of username
 *     links
 */
function _avatarapproval_get_users($md5) {
	$query = db_select('users', 'u');
  $query->innerJoin('avatar_approval', 'a', 'u.uid = a.uid');

  $query->fields('u', array('name', 'uid'))
  	->condition('a.md5', $md5, '=')
  	->condition('a.uid', 0, '!=')
  	->condition('a.active', AVATAR_ACTIVE, '=');


  $user_results = $query->execute();
  $results = array();
  foreach ($user_results as $user) {
    $results[$user->uid] = $user->name;
  }

  return theme('avatarapproval_get_name', array('user' => $results));
}
/**
 * Display administration page
 *
 * This function displays the avatar approval administration page.
 * It passes url arg so the form can decide what data it should display
 *
 * @param $arg (Optional)
 *     The section to display. Defaults to "" (empty string), which leads
 *     to awaiting approval section. Other values are "approve" and
 *     "disapprove", respectively
 */
function avatarapproval_admin_overview($arg = "") {
  // If user pictures are not enabled
  if (!variable_get('user_pictures', 0)) {
    if (user_access('access administration pages')) {
      $message = t('User pictures are not !enabled.',
      array('!enabled' => l(t('enabled'), 'admin/user/settings'))
      );
    }
    else {
      $message = t('User pictures are not enabled.');
    }
    drupal_set_message(filter_xss($message));
  }

  return drupal_get_form('avatarapproval_admin_overview_form', $arg);
}

/**
 * Generate the avatar approval settings form
 *
 * @param $form
 *     Will be initialized with the form fields
 * @param $form_state
 *     Not used
 * @return
 *     A system settings form that allows to modify all the
 *     module settings
 */
function avatarapproval_admin_settings_form($form, &$form_state) {
  $form = array();
  // Define fieldsets
  $form['awaitingpic_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Awaiting approval avatar settings'),
    '#collapsible' => TRUE,
  );
  $form['notifications_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Reminders settings'),
    '#collapsible' => TRUE,
  );

  $form['notifications_fieldset']['notifications_email_fieldset'] = array(
    '#type' => 'fieldset',
    '#weight' => 1,
    '#title' => t('Email notification settings'),
  );
  // Settings interface
  $form['awaitingpic_fieldset']['avatarapproval_use_awaiting_picture'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use a specific picture for users awaiting approval avatar'),
    '#description' => t('Choose either to use a specific picture for the avatar of the user who have submitted a picture for approval or to use the default avatar picture.'),
    '#default_value' => variable_get('avatarapproval_use_awaiting_picture', TRUE),
  );
  $form['awaitingpic_fieldset']['avatarapproval_awaiting_picture_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Awaiting approval avatar picture'),
    '#description' => t('Path of the avatar to use for the users who have submitted a picture for approval.'),
    '#default_value' => variable_get('avatarapproval_awaiting_picture_url', AVATAR_AWAITING_APPROVAL),
  );

  $form['notifications_fieldset']['avatarapproval_notify_moderator_login'] = array(
    '#type' => 'checkbox',
    '#title' => t('Notify pending avatar approvals on moderator login'),
    '#description' => t('If an user has the permission to approve avatars, notify that there are pending requests when he logs in.'),
    '#default_value' => variable_get('avatarapproval_notify_moderator_login', TRUE),
  );

  $form['notifications_fieldset']['avatarapproval_notify_moderator_email'] = array(
    '#type' => 'checkbox',
    '#title' => t('Notify pending avatar approvals to moderators by email'),
    '#description' => t('If an user has the permission to approve avatars, notify that there are pending requests by sending a periodical email.' .
                       'The period is variable and depends on a threshold. If the threshold is 3, the mail is sent when queue length is 3, 9, 27...'),
    '#default_value' => (variable_get('avatarapproval_notify_moderator_email', TRUE)),
  );
  $mail_token_help = t('Available variables are:').implode(", ", array_keys(avatarapproval_mail_tokens()));
  $form['notifications_fieldset']['notifications_email_fieldset']['avatarapproval_notify_moderator_email_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => filter_xss($mail_token_help),
    '#default_value' => _avatarapproval_mail_text('avatarapproval_notify_moderator_email_subject'),
    '#maxlength' => 180,
  );
  $form['notifications_fieldset']['notifications_email_fieldset']['avatarapproval_notify_moderator_email_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => filter_xss($mail_token_help),
    '#default_value' => _avatarapproval_mail_text('avatarapproval_notify_moderator_email_body'),
  );
  $form['notifications_fieldset']['notifications_email_fieldset']['avatarapproval_notify_moderator_email_threshold'] = array(
    '#type' => 'textfield',
    '#title' => t('Approval queue length threshold'),
    '#description' => t('Please specify how many requests must be queued prior to sending notification e-mails. If you set the value to 1, an e-mail is sent every new request.'),
    '#default_value' => variable_get('avatarapproval_notify_moderator_email_threshold', 3),
    '#maxlength' => 5,
    '#size' => 5,
  );

  $form['#submit'][] = "avatarapproval_admin_settings_form_submit";
  $form['#validate'][] = "avatarapproval_admin_settings_form_validate";

  return system_settings_form($form);
}

/**
 * Implements hook_form_submit().
 *
 * Submission handler for the avatarapproval admin settings form.
 */
function avatarapproval_admin_settings_form_submit(&$form, &$form_state) {
  // Change approval picture
  $oldpic = variable_get('avatarapproval_awaiting_picture_url', AVATAR_AWAITING_APPROVAL);
  $newpic = $form['#post']['avatarapproval_awaiting_picture_url'];
  if ($form['#post']['op'] == t("Reset to defaults")) {
    variable_set('avatarapproval_awaiting_picture_url', AVATAR_AWAITING_APPROVAL);
    $newpic = AVATAR_AWAITING_APPROVAL;
  }

  if ($newpic != $oldpic) {
    avatarapproval_refresh_awaiting_avatar($newpic);
  }
}

/**
 * Implements hook_form_validate().
 *
 * Validation handler for the avatarapproval admin settings form.
 */
function avatarapproval_admin_settings_form_validate(&$form, &$form_state) {

  // None of the following actions must be taken for a reset setting operation,
  // so we return
  if ($form['#post']['op'] == t("Reset to defaults")) {
    return;
  }

  if (!is_numeric($form['#post']['avatarapproval_notify_moderator_email_threshold']) ||
      $form['#post']['avatarapproval_notify_moderator_email_threshold'] <= 0) {
    form_set_error('avatarapproval_notify_moderator_email_threshold', t("Threshold value must be a positive integer number."));
  }

  $newpic = $form['#post']['avatarapproval_awaiting_picture_url'];
  if (!file_exists($newpic)) {
    form_set_error('avatarapproval_awaiting_picture_url', t("The selected file cannot be found, or there are insufficent permissions to access it."));
  }
}

/**
 * Generate the avatar approval moderation form
 *
 * This will display the operations commands to moderate avatars and a
 * list of selectable and sortable entries from the approval queues
 *
 * @param $form
 *     Will be initialized with the form fields
 * @param $form_state
 *     Not used
 * @param $type
 *     The currently selected section, used to filter the available
 *     operations
 * @return
 *     A system settings form that allows to modify all the
 *     module settings
 */
function avatarapproval_admin_overview_form($form, $form_state, $type) {
  // The beginings of the Update Options form
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  // Get the operations defined earlier in the _avatarapproval_operations function
  $operations = _avatarapproval_operations();
  // Unset the operation for the current screen since the avatar already has that state
  unset($operations[$type]);
  $options = array();
  foreach ($operations as $key => $value) {
    // Set each option
    $options[$key] = $value['text'];
  }

  // We create the options form
  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => 'approve',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  // Next we load the forms header
  $header = array(
    'picture' => array(
      'data' => t('Avatar'),
    ),
  	'users' => array(
      'data' => t('User'),
    ),
  	'timestamp' => array(
      'data' => t('Time'),
      'field' => 'timestamp',
      'sort' => 'desc',
    ),
  	'moderator' => array(
      'data' => t('Moderated by'),
      'field' => 'moderator',
      'sort' => 'desc',
    ),
  );

  // Set the basic SQL Statement
  $sql = db_select('{avatar_approval}', 'a')
  	->extend('TableSort')
    ->extend('PagerDefault')
  	->fields('a', array('md5', 'timestamp', 'moderator', 'fid'));

  $result = $sql->execute();

  // If $type is NULL, they are at the AVATAR_APPROVAL_PAGE.  The
  // default menu setting shows all avatars that need to be moderated.
  if ($type == "") {
    $sql = $sql->condition('approved', AVATAR_NOT_YET_MODERATED, '=');
  }
  elseif ($type == 'disapprove') {
    $sql = $sql->condition('approved', AVATAR_DISAPPROVED, '=');
  }
  elseif ($type == 'approve') {
    $sql = $sql->condition('approved', AVATAR_APPROVED, '=');
  }
	// Order by table header
  $sql = $sql
  	->orderByHeader($header)
  	->range(0,25);

  $result = $sql->execute();

  $avatars = array();
  foreach ($result as $avatar) {
    if (!$avatar->moderator) {
      $avatar->moderator = t('Pending moderation');
    }

    $file = file_load($avatar->fid);
    $avatars[$avatar->md5] = array(
      'picture' => theme('image_style',
        array(
    			'style_name' => variable_get('user_picture_style', ''),
        	'path' => $file->uri,
        	'title' => t('!uri', array('!uri' => $file->uri))
        )
      ),
      'users' => _avatarapproval_get_users($avatar->md5),
    	'timestamp' => format_date($avatar->timestamp, 'short'),
    	'moderator' => $avatar->moderator
    );
  }

  $form['avatars'] = array(
    '#type' => 'tableselect',
    '#options' => $avatars,
    '#header' => $header,
  	'#multiple' => TRUE,
  	'#empty' => t('No avatars found.')
  );
  $form['pager'] = array('#value' => theme('pager', array('tags' => NULL, 'element' => 0)));

  return $form;
}

/**
 * Implements hook_form_validate().
 *
 * The validation handler for the avatar approval admin form.
 */
function avatarapproval_admin_overview_form_validate($form, $form_state) {
  // If no boxes were checked, set an error
  if (count($form_state['values']['avatars']) == 0) {
    form_set_error('', t('Please select one or more avatars to perform the update on.'));
  }
}

/**
 * Implements hook_form_submit().
 *
 * The submit function for the avatar approval admin form.
 */
function avatarapproval_admin_overview_form_submit($form, $form_state) {
  $edit = $form_state['values'];
  // Get a list of operations
  $operations = _avatarapproval_operations();
  // Select the proper callback from the operation
  // ie: $operation['disapprove']['callback']
  if ($operations[$edit['operation']]['callback']) {
    $callback = $operations[$edit['operation']]['callback'];
    if(!is_null($callback)) {
      // Then run through all the avatars that were checked'
      foreach ($edit['avatars'] as $md5 => $value) {
        if (!empty($value)) {
          $callback($md5);
        }
      }
    }
  }
  cache_clear_all();
  $message = t('The update ('.check_plain($edit['operation']).') has been performed.');
  drupal_set_message(filter_xss($message));
}
