 <?php

/**
 * @file
 * Rules integration for the Avatar approval module.
 */

/**
 * (Internal) Helper function to get an user's avatar MD5
 *
 * @param $uid
 *   The id of the user to look for
 * @return an md5
 */
function _avatarapproval_rules_get_avatar_md5($uid) {
  return db_select('avatar_approval', 'a')
    ->fields('a', array('md5'))
    ->condition('uid', $uid, '=')
    ->condition('active', AVATAR_ACTIVE, '=')
    ->condition('approved', AVATAR_NOT_YET_MODERATED, '=')
    ->execute()
    ->fetchField();
}

/**
 * Implements hook_rules_event_info().
 *
 * @return
 * 		An associative array for the Rules API which defines the
 * 		following events:
 * 		- avatarapproval_rules_event_avatar_add: Triggered when a new avatar is
 * 			added to moderation queue
 * 		- avatarapproval_rules_event_avatar_approve: Triggered when an avatar is
 * 			approved
 * 		- avatarapproval_rules_event_avatar_disapprove: Triggered when an avatar
 * 			is disapproved
 */
function avatarapproval_rules_event_info() {
  $defaults = array(
    'variables' => array(
      'avatarapproval_user_uid' => array(
        'type' => 'integer',
        'label' => t('Avatar owner user id'),
        'description' => t('The id of the user to moderate.'),
      ),
      'avatarapproval_moderator_uid' => array(
        'type' => 'integer',
        'label' => t('Moderator user id'),
        'description' => t('The id of the user which is performing a moderation operation on an avatar.'),
      ),
      'avatarapproval_operation_name' => array(
        'type' => 'text',
        'label' => t('Operation name'),
        'description' => t('The name of the moderation operation.'),
      ),
    ),
    'group' => t('Avatar approval'),
  );
  return array(
    'avatarapproval_rules_event_avatar_add' => $defaults + array(
      'label' => t('A new avatar is added to moderation queue'),
      'help' => t('When an user changes his profile picture, the newly submitted avatar is added to the "Awaiting moderation" queue.'),
    ),
    'avatarapproval_rules_event_avatar_approve' => $defaults + array(
      'label' => t('An avatar is approved'),
      'help' => t('Any of the moderators approved an avatar from the moderation queues'),
    ),
    'avatarapproval_rules_event_avatar_disapprove' => $defaults + array(
      'label' => t('An avatar is disapproved'),
      'help' => t('Any of the moderators disapproved an avatar from the moderation queues'),
    ),
  );
}

/**
 * Implements hook_rules_action_info().
 */
function avatarapproval_rules_action_info() {
  $defaults = array(
    'parameter' => array(
      'avatarapproval_user_uid' => array(
        'label' => t('Avatar owner user id'),
        'type' => 'integer',
      ),
    ),
    'access callback' => 'avatarapproval_rules_integration_access',
  );

  return array(
    'avatarapproval_action_approve' => $defaults + array(
      'label' => t('Approve an user\'s avatar'),
      'group' => t('Avatar approval'),
      'base' => 'avatarapproval_rules_action_avatar_approve',
    ),
    'avatarapproval_action_disapprove' => $defaults + array(
      'label' => t('Disapprove an user\'s avatar'),
      'group' => t('Avatar approval'),
      'base' => 'avatarapproval_rules_action_avatar_disapprove',
    ),
    'avatarapproval_action_delete' => $defaults + array(
      'label' => t('Delete an user\'s avatar'),
      'group' => t('Avatar approval'),
      'base' => 'avatarapproval_rules_action_avatar_delete',
    ),
  );
}

/**
 * Action: Approve avatar.
 *
 * @param $uid
 * 		The id of the user whose avatar should be approved
 */
function avatarapproval_rules_action_avatar_approve($uid) {
  avatarapproval_do_approval(_avatarapproval_rules_get_avatar_md5($uid));
}

/**
 * Action: Disapprove avatar.
 *
 * @param $uid
 * 		The id of the user whose avatar should be disapproved
 */
function avatarapproval_rules_action_avatar_disapprove($uid) {
  avatarapproval_do_disapprove(_avatarapproval_rules_get_avatar_md5($uid));
}

/**
 * Action: Delete avatar.
 *
 * @param $uid
 * 		The id of the user whose avatar should be deleted
 */
function avatarapproval_rules_action_avatar_delete($uid) {
  avatarapproval_do_delete(_avatarapproval_rules_get_avatar_md5($uid));
}

/**
 * Avatar Approval Rules integration access callback.
 */
function avatarapproval_rules_integration_access($type, $name) {
  if ($name == "avatarapproval_rules_action_avatar_delete") {
    return user_access('delete avatars');
  } else if($name == "avatarapproval_rules_action_avatar_approve" || 
            $name == "avatarapproval_rules_action_avatar_disapprove") {
    user_access('moderate avatars');
  }
}
